use super::*;

pub struct DirectoryIterator<'a> {
    parent: &'a Directory,
    state:  DirectoryIteratorState<'a>
}

impl<'a> DirectoryIterator<'a> {
    pub fn new(parent: &Directory) -> DirectoryIterator {
        DirectoryIterator{parent, state: DirectoryIteratorState::Root}
    }
}

impl<'a> std::iter::Iterator for DirectoryIterator<'a> {
    type Item = DirectoryIteratorElement<'a>;

    fn next(&mut self) -> Option<DirectoryIteratorElement<'a>> {
        match std::mem::replace(&mut self.state, DirectoryIteratorState::Done) {
            DirectoryIteratorState::Root => {
                self.state = DirectoryIteratorState::Data(self.parent.data.iter());
                return Some(DirectoryIteratorElement::Root(&self.parent));
            }
            DirectoryIteratorState::Data(mut iter) => {
                if let Some(data) = iter.next() {
                    self.state = DirectoryIteratorState::Data(iter);
                    return Some(DirectoryIteratorElement::Data(data));
                }

                else {
                    self.state = DirectoryIteratorState::Directory(self.parent.dirs.iter());
                    return self.next();
                }
            },
            DirectoryIteratorState::Directory(mut iter) => {
                if let Some(dir) = iter.next() {
                    self.state = DirectoryIteratorState::Directory(iter);
                    return Some(DirectoryIteratorElement::Directory(dir));
                }

                else {
                    return self.next();
                }
            },
            DirectoryIteratorState::Done => {
                return None;
            }
        }
    }
}

enum DirectoryIteratorState<'a> {
    Root,
    Data(std::slice::Iter<'a, Data>),
    Directory(std::slice::Iter<'a, Directory>),
    Done
}

pub enum DirectoryIteratorElement<'a> {
    Root(&'a Directory),
    Data(&'a Data),
    Directory(&'a Directory),
}